@@ -44,10 +44,11 @@
 #include <QtCore/QJsonObject>
 #include <QtCore/QVersionNumber>
 
-#include <QtGui/QOpenGLTexture>
+#include <QOpenGLTexture>
 
 #include <Qt3DRender/QGeometry>
 #include <Qt3DRender/private/renderlogging_p.h>
+#include <Qt3DCore/private/qloadgltf_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -102,12 +103,7 @@ bool GLTFGeometryLoader::load(QIODevice *ioDev, const QString &subMesh)
 {
     Q_UNUSED(subMesh);
 
-    QByteArray jsonData = ioDev->readAll();
-    QJsonDocument sceneDocument = QJsonDocument::fromBinaryData(jsonData);
-    if (sceneDocument.isNull())
-        sceneDocument = QJsonDocument::fromJson(jsonData);
-
-    if (Q_UNLIKELY(!setJSON(sceneDocument))) {
+    if (Q_UNLIKELY(!setJSON(qLoadGLTF(ioDev->readAll())))) {
         qCWarning(GLTFGeometryLoaderLog, "not a JSON document");
         return false;
     }
@@ -139,7 +135,8 @@ GLTFGeometryLoader::BufferData::BufferData(const QJsonObject &json)
 }
 
 GLTFGeometryLoader::AccessorData::AccessorData()
-    : type(QAttribute::Float)
+    : bufferViewIndex(0)
+    , type(QAttribute::Float)
     , dataSize(0)
     , count(0)
     , offset(0)
@@ -303,11 +300,11 @@ void GLTFGeometryLoader::processJSONBufferView(const QString &id, const QJsonObj
     const auto &bufferData = *it;
 
     int target = json.value(KEY_TARGET).toInt();
-    Qt3DRender::QBuffer::BufferType ty(Qt3DRender::QBuffer::VertexBuffer);
 
     switch (target) {
-    case GL_ARRAY_BUFFER:           ty = Qt3DRender::QBuffer::VertexBuffer; break;
-    case GL_ELEMENT_ARRAY_BUFFER:   ty = Qt3DRender::QBuffer::IndexBuffer; break;
+    case GL_ARRAY_BUFFER:
+    case GL_ELEMENT_ARRAY_BUFFER:
+        break;
     default:
         qCWarning(GLTFGeometryLoaderLog, "buffer %ls unsupported target: %d",
                   qUtf16PrintableImpl(id), target);
@@ -330,7 +327,6 @@ void GLTFGeometryLoader::processJSONBufferView(const QString &id, const QJsonObj
     }
 
     Qt3DRender::QBuffer *b = new Qt3DRender::QBuffer();
-    b->setType(ty);
     b->setData(bytes);
     m_gltf1.m_buffers[id] = b;
 }
@@ -345,11 +341,10 @@ void GLTFGeometryLoader::processJSONBufferViewV2(const QJsonObject &json)
     const auto bufferData = m_gltf2.m_bufferDatas[bufferIndex];
 
     int target = json.value(KEY_TARGET).toInt();
-    Qt3DRender::QBuffer::BufferType ty(Qt3DRender::QBuffer::VertexBuffer);
-
     switch (target) {
-    case GL_ARRAY_BUFFER:           ty = Qt3DRender::QBuffer::VertexBuffer; break;
-    case GL_ELEMENT_ARRAY_BUFFER:   ty = Qt3DRender::QBuffer::IndexBuffer; break;
+    case GL_ARRAY_BUFFER:
+    case GL_ELEMENT_ARRAY_BUFFER:
+        break;
     default:
         return;
     }
@@ -368,7 +363,7 @@ void GLTFGeometryLoader::processJSONBufferViewV2(const QJsonObject &json)
                   qUtf16PrintableImpl(bufferData.path));
     }
 
-    Qt3DRender::QBuffer *b(new Qt3DRender::QBuffer(ty));
+    auto b = new Qt3DRender::QBuffer;
     b->setData(bytes);
     m_gltf2.m_buffers.push_back(b);
 }
